"""
    Copyright (C) 2021-present, Murdo B. Maclachlan

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program. If not, see <https://www.gnu.org/licenses/>.
    
    Contact me at murdo@maclachlans.org.uk
"""

import praw
from time import sleep
from .misc import checkMessage, getCredentials

def tadr(Log, messageIDs, Static):
    
    
    Log.new("Initialising Reddit instance...")
    reddit = praw.Reddit(
        user_agent=Static.OS+":tor-auto-done-replier:v"+Static.VERSION+" (by /u/MurdoMaclachlan)",
        **getCredentials(Log, Static)
    )
    
    # Main program loop
    while True:
        
        # Fetch messages
        Log.new("Checking messages...")
        for message in reddit.inbox.comment_replies(limit=Static.LIMIT):
            
            # Main check, replying to message if necessary
            # The one second delay should ensure only 1 reply is ever needed
            if checkMessage(Log, message, messageIDs, Static):
                Log.new(f"Replying to message at: {message.context}")
                if Static.VERBOSE: Log.Notify.Notification.new(f"Replying to message at: {message.id}").show()
                sleep(1)
                message.reply(Static.REPLY)
        
        # Extra logs / functions depending on settings
        if Static.DEBUG: Log.new(f"Messages checked so far: \n{messageIDs}")
        if not Log.failed and Static.LOG_UPDATES: Log.new("Updating log..."); Log.update(Static)
        Log.new(f"Finished checking messages, waiting {Static.SLEEP} seconds.")
        
        sleep(Static.SLEEP)



"""
    Copyright (C) 2021-present, Murdo B. Maclachlan

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program. If not, see <https://www.gnu.org/licenses/>.
    
    Contact me at murdo@maclachlans.org.uk
"""

from configparser import ConfigParser
from datetime import datetime
from typing import Dict, List

def checkMessage(Log: object, message: object, messageIDs: List, Static: object) -> bool:
    
    # Avoid checking messages from before program start, or that have already been checked
    if message.created_utc < Static.START_TIME or message.id in messageIDs:
        return False
    
    messageIDs.append(message.id)
    
    if message.body.split(Static.SPLITTER)[0] in Static.MESSAGES and message.author.name in Static.AUTHORS:
        
        # Declaring these variables saves on API requests and speeds up program a lot.
        # They'll be deleted later on to ensure memory is saved, because I don't know
        # whether or not Python does that for you.
        parent = message.parent()
        parentBody = parent.body.casefold()
        
        # Haven't tried re-replying; try.
        if parentBody == "done":
            del parent, parentBody
            return True
        
        # Have tried re-replying; there's a problem.
        elif parentBody == Static.REPLY:
            Log.Notify.Notification.new(f"Problematic post found.").show()
            Log.new(Log.warning(f"Problematic post at: {parent.url}"))
            del parent, parentBody
            return False

# Use configparser magic to get the credentials from praw.ini
def getCredentials(Log: object, Static: object) -> Dict:
    
    # Read config section titled "adr" and return details
    try:
        credentials = ConfigParser()
        credentials.read(f"{Static.PATHS['config']}/praw.ini")
        return dict(credentials["tadr"])
    
    # If praw.ini is missing, warn the user
    except (FileNotFoundError, KeyError):
        Log.new(Log.warning("praw.ini not found; please set up praw.ini and then re-run TADR. See README.md for instructions."))
        exit()

# Returns current time in human readable format
def getTime(timeToFind: float) -> str:
    return datetime.fromtimestamp(timeToFind).strftime("%Y-%m-%d %H:%M:%S")



"""
    Copyright (C) 2021-present, Murdo B. Maclachlan

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program. If not, see <https://www.gnu.org/licenses/>.
    
    Contact me at murdo@maclachlans.org.uk
"""

from colored import fg, attr
from os import environ, mkdir
from os.path import expanduser, isdir
from sys import platform
from time import time
from typing import List, NoReturn
from .misc import getTime

global VERSION
VERSION = "1.0.0"

# For console logging
class Log:
    
    def __init__(self, Notify: object) -> NoReturn:
        self.__log = []
        self.ConsoleColours = self.Colours(130, 0)
        self.failed = False
        self.Notify = Notify
        self.Notify.init("Auto Done Replier")
        
    class Colours:
            
        def __init__(self: object, warning: int, reset: int) -> NoReturn:
            self.RESET = attr(reset)
            self.WARNING = fg(warning)
    
    # Log to console, all that jazz
    def new(self, message: str) -> NoReturn:
        message = f"{getTime(time())} - {message}"
        self.__log.append(message)
        print(message)
    
    # Tries to update log.txt file; if it can't find the data directory,
    # it'll make it. On any unexpected exception, it'll disable future
    # log updates and spit the error to the console.
    def update(self, Static: object) -> NoReturn:
        if not self.failed:
            try:
                if not isdir(Static.PATHS["data"]):
                    self.new("data directory not found, it will be created.")
                    mkdir(Static.PATHS["data"])
                with open(f"{Static.PATHS['data']}/log.txt", "a") as file:
                    for i in self.__log:
                        file.write(f"{i}\n")
            except Exception as e:
                self.new(f"Failed to update log with error:\n{e}")
                self.failed = True
    
    # Returns a string that will appear coloured when printed to the console
    def warning(self, message: str) -> str:
        return self.ConsoleColours.WARNING + "WARNING: " + message + self.ConsoleColours.RESET

# Contains all static vars
class Static:
    
    def __init__(self, Log) -> NoReturn:
        self.AUTHORS = ["transcribersofreddit"]
        self.DEBUG = False
        self.LIMIT = 10
        self.LOG_UPDATES = False
        self.MESSAGES = ["Sorry; I can't find your transcript post on the link"]
        self.OS = platform
        self.PATHS = self.definePaths(expanduser("~"), self.OS, Log)
        self.REPLY = "done -- this was an automated action; please contact me with any questions."
        self.SLEEP = 10
        self.SPLITTER = "."
        self.START_TIME = time()
        self.VERBOSE = True
        self.VERSION = VERSION
    
    # Defines save paths for config and data based on the user's OS    
    def definePaths(self, home: str, os: str, Log: object) -> List:
        
        # Gets first 3 characters of OS
        os = ''.join(list(os)[:3])
        
        if os in ["dar", "lin", "win"]:
            
            # Windows is fucking stupid why would you use backslashes
            paths = {
                "config": environ["APPDATA"] + "\\tadr",
                "data": environ["APPDATA"] + "\\tadr\data"
            } if os == "win" else {
                "config": home + "/.config/tadr",
                "data": home + "/.tadr/data"
            }
                
            #Create any missing paths/directories
            for path in paths:
                if not isdir(paths[path]):
                    Log.new(f"Making path: {paths[path]}")
                    for directory in paths[path].split("/")[1:]:
                       if not isdir(paths[path].split(directory)[0] + directory):
                           Log.new(f"Making directory: {paths[path].split(directory)[0]}{directory}")
                           mkdir(paths[path].split(directory)[0] + directory)
            return paths
        
        # Exit is OS is unsupported
        else:
            Log.new(Log.warning(f"Unsupported operating system: {os}, exiting."))
            exit()



from setuptools import setup, find_packages
from core.classes import VERSION

def readme():
    return open('README.md', 'r').read()

setup(
    name="tadr",
    version=VERSION,
    scripts=["tadr"],
    author="Murdo Maclachlan",
    author_email="murdo@maclachlans.org.uk",
    description="A tool that will automatically reply done to the first 'cannot find transcription' message from the r/TranscribersOfReddit bot. ",
    long_description=readme(),
    long_description_content_type="text/markdown",
    url="https://github.com/MurdoMaclachlan/tadr",
    packages=find_packages(),
    install_requires=[
        "colored",
        "configparser",
        "praw>=7.1.2"
    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)",
        "Operating System :: OS Independent",
    ],
    license='GPLv3+'
)



"""
    Copyright (C) 2021-present, Murdo B. Maclachlan

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program. If not, see <https://www.gnu.org/licenses/>.
    
    Contact me at murdo@maclachlans.org.uk
"""

# CLASSES
from .classes import Log
from .classes import Static

# MAIN
from .main import tadr

# MISC
from .misc import checkMessage
from .misc import getCredentials
from .misc import getTime
